from typing import Optional

from kombu import producers, Exchange

from hijiki.broker.hijiki_broker import HijikiBroker


class Publisher():

    def __init__(self, host, username, password, port, cluster_hosts: str = None, heartbeat: Optional[float] = 60):
        self.client = HijikiBroker('client', host, username, password, port, cluster_hosts, heartbeat=heartbeat)

    def publish_message(self, event_name: str, data: str):
        payload = {"value": data}
        connection = self.client.get_celery_broker().broker_connection(heartbeat=self.client.heartbeat)
        with producers[connection].acquire(block=True) as producer:
            task_exchange = Exchange(event_name, type='topic')
            producer.publish(payload,  exchange=task_exchange, declare=[task_exchange], routing_key='x')



import time
from celery import Celery
from typing import Optional

from kombu import Connection, Exchange, Queue, Consumer

from hijiki.broker.broker_data import get_broker_url, init_os_environ


class HijikiBroker:
    celery_broker = None
    heartbeat = None

    def __init__(self, app_name, host, username, password, port, cluster_hosts, heartbeat: Optional[float] = 60):
        init_os_environ(host, username, password, port, cluster_hosts)
        self.heartbeat = heartbeat
        self.celery_broker = Celery(app_name, broker=get_broker_url(), set_as_current=True)


    def get_celery_broker(self):
        return self.celery_broker

    def ping(self):
        try:
            success_ping = False
            rabbit_url = get_broker_url()
            conn = Connection(rabbit_url)
            try:
                exchange = Exchange("ping-exchange", type="direct")
                queue = Queue(name="ping-queue", exchange=exchange, routing_key="Pong")

                def process_message(body, message):
                    print("The body is {}".format(body))
                    message.ack()

                with Consumer(conn, queues=queue, callbacks=[process_message], accept=["text/plain"]):
                    success_ping = True
            finally:
                conn.close()
            return success_ping
        except Exception as e:
            print("Failed to start broker")
            print(e)
            return False



import time
import unittest

from hijiki.publisher.Publisher import Publisher
from tests.runner import Runner

DLQ_RETRY_NUMBER = 11 # At eleven interaction the message goes to the Dead Letter Queue
SECS_TO_AWAIT_BROKER = 5
DLQ_DONT_RECEIVE_ERROR_WITH_AUTO_ACK_ENABLED = 1


class TestPublisherConsumer(unittest.TestCase):
    runner = None

    def setUp(self):
        self.runner = Runner()
        self.runner.run()
        self.pub = Publisher("localhost", "user", "pwd", 5672, heartbeat=30)
        self.NUMBER_OF_QUEUED_MESSAGES = 2

    def tearDown(self):
        self.runner.stop()

    def test_publish_a_message(self):
        self.runner.clear_results()
        self.pub.publish_message('teste1_event', '{"value": "This is the message"}')

    def test_consume_a_message(self):
        self.runner.clear_results()
        time.sleep(SECS_TO_AWAIT_BROKER)
        self.pub.publish_message('teste1_event', '{"value": "This is the message"}')
        time.sleep(SECS_TO_AWAIT_BROKER)
        self.assertEqual(len(self.runner.get_results()), 1)

    def test_consume_a_message_failed(self):
        self.runner.clear_results()
        time.sleep(SECS_TO_AWAIT_BROKER)
        self.pub.publish_message('erro_event', '{"value": "This is the message"}')
        time.sleep(SECS_TO_AWAIT_BROKER)
        self.assertEqual(DLQ_RETRY_NUMBER, len(self.runner.get_results()))

    def test_consume_a_message_failed_with_auto_ack_dont_go_to_DLQ(self):
        self.runner.clear_results()
        self.runner.set_auto_ack(True)
        try:
            time.sleep(SECS_TO_AWAIT_BROKER)
            self.pub.publish_message('erro_event', '{"value": "This is the message"}')
            time.sleep(SECS_TO_AWAIT_BROKER)
            self.assertEqual(DLQ_DONT_RECEIVE_ERROR_WITH_AUTO_ACK_ENABLED, len(self.runner.get_results()))
        finally:
            self.runner.set_auto_ack(False)

    def test_consume_a_message_dlq(self):
        self.runner.clear_results()
        time.sleep(SECS_TO_AWAIT_BROKER)
        for number in range(self.NUMBER_OF_QUEUED_MESSAGES):
            self.pub.publish_message(
                'erro_event',
                f'{{"value": "This is message number {number} that will be sent to dlq"}}'
            )

        time.sleep(SECS_TO_AWAIT_BROKER)
        self.assertEqual(self.NUMBER_OF_QUEUED_MESSAGES, len(self.runner.get_results_dlq()))

    def test_consume_a_message_without_consumer_dlq(self):
        self.runner.clear_results()
        time.sleep(SECS_TO_AWAIT_BROKER)
        for number in range(self.NUMBER_OF_QUEUED_MESSAGES):
            self.pub.publish_message(
                'without_dlq',
                '{"value": "This is the message that will fall into a dlq queue, which has no consumer"}'
            )
        time.sleep(SECS_TO_AWAIT_BROKER)
        self.assertEqual(
            self.NUMBER_OF_QUEUED_MESSAGES * DLQ_RETRY_NUMBER,
            len(self.runner.get_results())
        )





import os

BROKER_PORT = "BROKER_PORT"
BROKER_PWD = "BROKER_PWD"
BROKER_USERNAME = "BROKER_USERNAME"
BROKER_SERVER = "BROKER_SERVER"
BROKER_CLUSTER_SERVER = "BROKER_CLUSTER_SERVER"


def __build_cluster_uri(cluster_server: str, username: str, pwd: str):
    servers = cluster_server.split(',')
    uri = ""
    for s in servers:
        uri += f'amqp://{username}:{pwd}@{s};'
    return uri


def get_broker_url():
    cluster_server = os.environ[BROKER_CLUSTER_SERVER] if BROKER_CLUSTER_SERVER in os.environ else None
    server = os.environ[BROKER_SERVER] if BROKER_SERVER in os.environ else None
    username = os.environ[BROKER_USERNAME] if BROKER_USERNAME in os.environ else None
    pwd = os.environ[BROKER_PWD] if BROKER_PWD in os.environ else None
    port = os.environ[BROKER_PORT] if BROKER_PORT in os.environ else "5672"
    if cluster_server:
        return __build_cluster_uri(cluster_server, username, pwd)
    else:
        return f'amqp://{username}:{pwd}@{server}:{port}' if server else 'amqp://rabbitmq:rabbitmq@localhost:5672'


def init_os_environ(host, username, password, port, cluster_servers):
    if cluster_servers:
        os.environ[BROKER_CLUSTER_SERVER] = cluster_servers
    if host:
        os.environ[BROKER_SERVER] = host
    if username:
        os.environ[BROKER_USERNAME] = username
    if password:
        os.environ[BROKER_PWD] = password
    if port:
        os.environ[BROKER_PORT] = str(port)
